import React from 'react';
import { AsyncStorage } from 'react-native';
import { Subtract } from 'utility-types';

const storageKey = '@onboarding-switch/onboarding-status';

const enum onboardingState {
  done = 'done',
  pending = 'pending'
}

export interface InjectedOnboardingProps {
  onDone: () => Promise<void>;
}

interface State {
  onboarding?: onboardingState;
}

/**
 * A Higher Order Component that shows an Onboarding scene at the first app start (and again if onboarding was canceled)
 * The Onboarding Component will receive an {@code onDone} function, that receives no arguments and is to be called when
 * the onboarding scene should never be shown again
 *
 * @param App {React.ComponentType} the main app to show, once the onboarding is completed
 * @param Onboarding {React.ComponentType} the slideshow for onboarding
 * @param Loading {React.ComponentType} a loading screen to show while storage is read
 * @returns {React.ComponentClass}
 */
const withOnboarding =
  <P extends InjectedOnboardingProps>
    (App: React.ComponentType,
      Onboarding: React.ComponentType<P>,
      Loading: React.ComponentType,
  ) =>

    class WithOnboarding extends React.Component<Subtract<P, InjectedOnboardingProps>, State> {

      public constructor(props: any) {
        super(props);
        this.state = {};
      }

      public async onDone() {
        await AsyncStorage.setItem(storageKey, onboardingState.done);
        this.setState({ onboarding: onboardingState.done });
      }

      public async componentDidMount() {
        try {
          const value = await AsyncStorage.getItem(storageKey);
          if (!value) {
            this.setState({ onboarding: onboardingState.pending });
            await AsyncStorage.setItem(storageKey, onboardingState.pending);
          } else {
            this.setState({ onboarding: value as onboardingState });
          }
        } catch (err) {

        }
      }

      public render() {
        switch (this.state.onboarding) {
          case onboardingState.done:
            return (<App {...this.props} />);
          case onboardingState.pending:
            // @ts-ignore
            return (<Onboarding onDone={this.onDone.bind(this)} />);
          default:
            return (<Loading />);
        }
      }


    };

export default withOnboarding;

/**
 * Resets the onboarding status of the device. The next time the 
 * user opens the app, the onboarding screen will be presented.
 * 
 * @returns {Promise<void>} a promise the resolves upon 
 * successful deletion of the key
 */
export async function resetOnboarding() {
  await AsyncStorage.removeItem(storageKey);
}
